=== modified file 'upstream_import.py'
--- upstream_import.py	2006-05-18 16:22:20 +0000
+++ upstream_import.py	2006-05-18 16:36:49 +0000
@@ -1,5 +1,6 @@
 """Import upstream source into a branch"""
 
+import errno
 import os
 from shutil import rmtree
 from StringIO import StringIO
@@ -40,7 +41,7 @@
     return possible_prefix
 
 
-def do_directory(tt, trans_id, relative_path, path):
+def do_directory(tt, trans_id, tree, relative_path, path):
     if isdir(path) and tree.path2id(relative_path) is not None:
         tt.cancel_deletion(trans_id)
     else:
@@ -62,13 +63,15 @@
     """
     tar_file = tarfile.open('lala', 'r', tar_input)
     prefix = common_directory(tar_file.getnames())
-    added = []
     tt = TreeTransform(tree)
 
+    removed = set()
     for path, entry in tree.inventory.iter_entries():
         trans_id = tt.trans_id_tree_path(path)
         tt.delete_contents(trans_id)
+        removed.add(path)
 
+    added = set() 
     implied_parents = set()
     for member in tar_file.getmembers():
         relative_path = member.name 
@@ -78,37 +81,38 @@
             continue
         add_implied_parents(implied_parents, relative_path)
         trans_id = tt.trans_id_tree_path(relative_path)
-        added.append(relative_path)
+        added.add(relative_path.rstrip('/'))
         path = tree.abspath(relative_path)
         if member.isreg():
             tt.create_file(file_iterator(tar_file.extractfile(member)), 
                            trans_id)
         elif member.isdir():
-            do_directory(tt, trans_id, relative_path, path)
+            do_directory(tt, trans_id, tree, relative_path, path)
         elif member.issym():
             tt.create_symlink(member.linkname, trans_id)
 
-    for relative_path in implied_parents.difference(set(added)):
+    for relative_path in implied_parents.difference(added):
         if relative_path == "":
             continue
         trans_id = tt.trans_id_tree_path(relative_path)
         path = tree.abspath(relative_path)
-        do_directory(tt, trans_id, relative_path, path)
-        added.append(relative_path)
+        do_directory(tt, trans_id, tree, relative_path, path)
+        added.add(relative_path)
 
     for conflict in cook_conflicts(resolve_conflicts(tt), tt):
         warning(conflict)
     tt.apply()
-    update_ids(tree, added)
-
-
-def update_ids(tree, added):
+    update_ids(tree, added, removed)
+
+
+def update_ids(tree, added, removed):
     """Make sure that all present files files have file_ids.
     """
     # XXX detect renames
-    for path in sorted(added):
-        if tree.path2id(path) is None:
-            tree.add(path)
+    new = added.difference(removed)
+    deleted = removed.difference(added)
+    tree.add(sorted(new))
+    tree.remove(sorted(deleted, reverse=True))
 
 
 def do_import(source, tree_directory=None):
@@ -132,7 +136,7 @@
             source.endswith('.tar.bz2')):
             try:
                 tar_input = file(source, 'rb')
-            except:
+            except IOError, e:
                 if e.errno == errno.ENOENT:
                     raise NoSuchFile(source)
             try:

